@@ -17,12 +17,10 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
+#ifndef BACKENDS_CONFIG_H
+#define BACKENDS_CONFIG_H 1
 
 #include "parsing/config.h"
-#include "compat.h"
-#include "swftypes.h"
 
 namespace lightspark
 {
@@ -30,7 +28,7 @@ namespace lightspark
 	{
 	private:
 		ConfigParser* parser;
-		
+
 		void handleEntry();
 
 		//-- CONFIGURATION FILENAME AND SEARCH DIRECTORIES
@@ -75,4 +73,4 @@ namespace lightspark
 	};
 }
 
-#endif
+#endif /* BACKENDS_CONFIG_H */
