@@ -18,16 +18,17 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#ifndef WINMMPLUGIN_H
-#define WINMMPLUGIN_H
+#ifndef BACKENDS_INTERFACES_AUDIO_WINMM_WINMMPLUGIN_H
+#define BACKENDS_INTERFACES_AUDIO_WINMM_WINMMPLUGIN_H 1
 
 #include <iostream>
 #include <windows.h>
+#undef RGB //conflicts with swftypes.h
 #include <mmsystem.h>
-#include "../IAudioPlugin.h"
-#include "../../../decoder.h"
-#include "../../../../compat.h"
-#include "../../../../threading.h"
+#include "backends/interfaces/audio/IAudioPlugin.h"
+#include "backends/decoder.h"
+#include "compat.h"
+#include "threading.h"
 
 namespace lightspark
 {
@@ -79,4 +80,4 @@ public:
 };
 
 }
-#endif
+#endif /* BACKENDS_INTERFACES_AUDIO_WINMM_WINMMPLUGIN_H */
