@@ -17,10 +17,10 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#ifndef _RTMP_UTILS_H
-#define _RTMP_UTILS_H
+#ifndef BACKENDS_RTMPUTILS_H
+#define BACKENDS_RTMPUTILS_H 1
 
-#include "netutils.h"
+#include "backends/netutils.h"
 
 namespace lightspark
 {
@@ -38,4 +38,4 @@ public:
 };
 
 };
-#endif
+#endif /* BACKENDS_RTMPUTILS_H */
