@@ -210,8 +210,15 @@ void SecurityManager::loadPolicyFile(std::multimap<tiny_string, T*>& pendingFile
 	if(pendingFiles.count(file->getURL().getHostname()) > 0)
 	{
 		LOG(LOG_INFO, _("SECURITY: Loading policy file (") << file->getURL() << ")");
+
+		// Policy files are downloaded in blocking manner,
+		// release the lock during loading
+		l.release();
+
 		file->load();
 
+		l.acquire();
+
 		std::pair< typename std::multimap<tiny_string, T*>::iterator, typename std::multimap<tiny_string, T*>::iterator > range;
 		range = pendingFiles.equal_range(file->getURL().getHostname());
 		typename std::multimap<tiny_string, T*>::iterator i;
@@ -271,9 +278,9 @@ std::list<T*> *SecurityManager::searchPolicyFiles(const URLInfo& url,
 	{
 		LOG(LOG_INFO, _("SECURITY: Master policy file is loaded and valid (") << url << ")");
 
-		PolicySiteControl::METAPOLICY siteControl = master->getSiteControl()->getPermittedPolicies();
+		PolicyFile::METAPOLICY siteControl = master->getMetaPolicy();
 		//Master defines no policy files are allowed at all
-		if(siteControl == PolicySiteControl::NONE)
+		if(siteControl == PolicyFile::NONE)
 		{
 			LOG(LOG_INFO, _("SECURITY: DISALLOWED: Master policy file disallows policy files"));
 			delete result;
@@ -283,7 +290,7 @@ std::list<T*> *SecurityManager::searchPolicyFiles(const URLInfo& url,
 		result->push_back(master);
 
 		//Non-master policy files are allowed
-		if(siteControl != PolicySiteControl::MASTER_ONLY)
+		if(siteControl != PolicyFile::MASTER_ONLY)
 		{
 			LOG(LOG_INFO, _("SECURITY: Searching for loaded non-master policy files (") <<
 					loadedFiles.count(url.getHostname()) << ")");
@@ -388,9 +395,7 @@ SocketPFileList* SecurityManager::searchSocketPolicyFiles(const URLInfo& url, bo
 	//The target port is checked last if allowed
 	if(master->isLoaded() && master->isValid() && (result != NULL))
 	{
-		PolicySiteControl::METAPOLICY siteControl = master->getSiteControl()->getPermittedPolicies();
-
-		if (siteControl == PolicySiteControl::ALL)
+		if (master->getMetaPolicy() == PolicyFile::ALL)
 		{
 			SocketPolicyFile* destination = getSocketPolicyFileByURL(url);
 			if (destination == NULL)
@@ -761,6 +766,9 @@ PolicyFile::~PolicyFile()
 	for(list<PolicyAllowAccessFrom*>::iterator i = allowAccessFrom.begin();
 			i != allowAccessFrom.end(); ++i)
 		delete (*i);
+
+	if (siteControl)
+		delete siteControl;
 }
 
 /**
@@ -777,8 +785,6 @@ void PolicyFile::load()
 	//Invalid URLPolicyFile or already loaded, ignore this call
 	if(!isValid() || isLoaded())
 		return;
-	//We only try loading once, if something goes wrong, valid will be set to 'invalid'
-	loaded = true;
 	
 	ignore = isIgnoredByMaster();
 
@@ -789,6 +795,16 @@ void PolicyFile::load()
 
 	Mutex::Lock l(mutex);
 
+	if (isLoaded())
+	{
+		// Another thread already processed this PolicyFile
+		// while we were downloading
+		return;
+	}
+
+	//We only try loading once, if something goes wrong, valid will be set to 'invalid'
+	loaded = true;
+
 	//We've checked the master file to see of we need to ignore this file. (not the case)
 	//Now let's parse this file.
 	if(isValid() && !isIgnored())
@@ -807,7 +823,7 @@ void PolicyFile::load()
 			//No more parsing is needed if this site-control entry specifies that
 			//no policy files are allowed
 			if (elementType == CrossDomainPolicy::SITE_CONTROL && isMaster() && 
-			    siteControl->getPermittedPolicies() == PolicySiteControl::NONE)
+			    getMetaPolicy() == PolicyFile::NONE)
 			{
 				break;
 			}
@@ -821,12 +837,8 @@ void PolicyFile::load()
 
 		if(isMaster())
 		{
-			//Set siteControl to the default value if it isn't set before and we are a master file
-			if(siteControl == NULL)
-				siteControl = new PolicySiteControl(this, "");
-
 			//Ignore this file if the site control policy is "none"
-			if(siteControl->getPermittedPolicies() == PolicySiteControl::NONE)
+			if(getMetaPolicy() == PolicyFile::NONE)
 				ignore = true;
 
 			valid = checkSiteControlValidity();
@@ -848,7 +860,18 @@ void PolicyFile::handlePolicyElement(CrossDomainPolicy::ELEMENT& elementType,
 {
 	if(elementType == CrossDomainPolicy::SITE_CONTROL)
 	{
-		siteControl = new PolicySiteControl(this, parser.getPermittedPolicies());
+		if (siteControl)
+		{
+			// From spec: If site-control is defined
+			// multiple times, "none" takes preference.
+			if (getMetaPolicy() == NONE)
+				return;
+
+			delete siteControl;
+			siteControl = NULL;
+		}
+
+		siteControl = new PolicySiteControl(parser.getPermittedPolicies());
 	}
 	else if(elementType == CrossDomainPolicy::ALLOW_ACCESS_FROM)
 	{
@@ -857,6 +880,30 @@ void PolicyFile::handlePolicyElement(CrossDomainPolicy::ELEMENT& elementType,
 	}
 }
 
+PolicyFile::METAPOLICY PolicyFile::getMetaPolicy()
+{
+	if (!isMaster())
+	{
+		// Only the master can have site-control
+		return PolicyFile::NONE;
+	}
+	else if (siteControl)
+	{
+		return siteControl->getPermittedPolicies();
+	}
+	else if (isValid() && isLoaded())
+	{
+		// Policy file is loaded but does not contain
+		// site-control tag
+		return PolicySiteControl::defaultSitePolicy(getType());
+	}
+	else
+	{
+		// Policy file is invalid, deny everything
+		return PolicyFile::NONE;
+	}
+}
+
 /**
  * \brief Constructor for URLPolicyFile
  *
@@ -920,7 +967,7 @@ URLPolicyFile* URLPolicyFile::getMasterPolicyFile()
  *
  * \return \c true if this policy file is a master policy file, otherwise \c false
  */
-bool URLPolicyFile::isMaster()
+bool URLPolicyFile::isMaster() const
 {
 	return url.getPath() == "/crossdomain.xml";
 }
@@ -933,14 +980,14 @@ bool URLPolicyFile::isIgnoredByMaster()
 		URLPolicyFile* master = getMasterPolicyFile();
 		getSys()->securityManager->loadURLPolicyFile(master);
 		//Master policy file found and valid and has a site-control entry
-		if(master->isValid() && master->getSiteControl() != NULL)
+		if(master->isValid())
 		{
-			PolicySiteControl::METAPOLICY permittedPolicies = master->getSiteControl()->getPermittedPolicies();
+			PolicyFile::METAPOLICY permittedPolicies = master->getMetaPolicy();
 			//For all types: master-only, none
-			if(permittedPolicies == PolicySiteControl::MASTER_ONLY || permittedPolicies == PolicySiteControl::NONE)
+			if(permittedPolicies == PolicyFile::MASTER_ONLY || permittedPolicies == PolicyFile::NONE)
 				return true;
 			//Only for FTP: by-ftp-filename
-			else if(subtype == FTP && permittedPolicies == PolicySiteControl::BY_FTP_FILENAME && 
+			else if(subtype == FTP && permittedPolicies == PolicyFile::BY_FTP_FILENAME && 
 					url.getPathFile() != "crossdomain.xml")
 				return true;
 		}
@@ -993,9 +1040,9 @@ bool URLPolicyFile::retrievePolicyFile(vector<unsigned char>& outData)
 		//If the site-control policy of the master policy file is by-content-type, only policy files with
 		//content-type = text/x-cross-domain-policy are allowed.
 		URLPolicyFile* master = getMasterPolicyFile();
-		if(master->isValid() && master->getSiteControl() != NULL &&
+		if(master->isValid() &&
 				(subtype == HTTP || subtype == HTTPS) &&
-				master->getSiteControl()->getPermittedPolicies() == PolicySiteControl::BY_CONTENT_TYPE &&
+				master->getMetaPolicy() == PolicyFile::BY_CONTENT_TYPE &&
 				contentType != "text/x-cross-domain-policy")
 		{
 			LOG(LOG_INFO, _("SECURITY: Policy file content-type isn't strict, marking invalid"));
@@ -1034,11 +1081,11 @@ bool URLPolicyFile::checkSiteControlValidity()
 {
 	//by-ftp-filename only applies to FTP
 	if((subtype == HTTP || subtype == HTTPS) && 
-	   siteControl->getPermittedPolicies() == PolicySiteControl::BY_FTP_FILENAME)
+	   getMetaPolicy() == PolicyFile::BY_FTP_FILENAME)
 		return false;
 	//by-content-type only applies to HTTP(S)
 	else if(subtype == FTP && 
-		siteControl->getPermittedPolicies() == PolicySiteControl::BY_CONTENT_TYPE)
+		getMetaPolicy() == PolicyFile::BY_CONTENT_TYPE)
 		return false;
 
 	return true;
@@ -1047,11 +1094,11 @@ bool URLPolicyFile::checkSiteControlValidity()
 /**
  * \brief Checks whether this policy file allows the given URL access
  *
- * \param url The URL to check if it is allowed by the policy file
- * \param to The URL that is being requested by a resource at \c url
+ * \param requestingUrl The URL (of the SWF file) that is requesting the data
+ * \param to The URL that is being requested by a resource at \c requestingUrl
  * \return \c true if allowed, otherwise \c false
  */
-bool URLPolicyFile::allowsAccessFrom(const URLInfo& url, const URLInfo& to)
+bool URLPolicyFile::allowsAccessFrom(const URLInfo& requestingUrl, const URLInfo& to)
 {
 	//File must be loaded
 	if(!isLoaded())
@@ -1069,7 +1116,7 @@ bool URLPolicyFile::allowsAccessFrom(const URLInfo& url, const URLInfo& to)
 	for(; i != allowAccessFrom.end(); ++i)
 	{
 		//This allow-access-from entry applies to our domain AND it allows our domain
-		if((*i)->allowsAccessFrom(url))
+		if((*i)->allowsAccessFrom(requestingUrl))
 			return true;
 	}
 	return false;
@@ -1140,11 +1187,11 @@ SocketPolicyFile::SocketPolicyFile(const URLInfo& _url):
 /**
  * \brief Checks whether this policy file allows the given URL access
  *
- * \param url The URL to check if it is allowed by the policy file
- * \param to The URL that is being requested by a resource at \c url
+ * \param requestingUrl The URL (of the SWF file) that is requesting the data
+ * \param to The URL that is being requested by a resource at \c requestingUrl
  * \return \c true if allowed, otherwise \c false
  */
-bool SocketPolicyFile::allowsAccessFrom(const URLInfo& url, const URLInfo& to)
+bool SocketPolicyFile::allowsAccessFrom(const URLInfo& requestingUrl, const URLInfo& to)
 {
 	//File must be loaded
 	if(!isLoaded())
@@ -1158,7 +1205,7 @@ bool SocketPolicyFile::allowsAccessFrom(const URLInfo& url, const URLInfo& to)
 	for(; i != allowAccessFrom.end(); ++i)
 	{
 		//This allow-access-from entry applies to our domain AND it allows our domain
-		if((*i)->allowsAccessFrom(url, to.getPort()))
+		if((*i)->allowsAccessFrom(requestingUrl, to.getPort()))
 			return true;
 	}
 	return false;
@@ -1169,7 +1216,7 @@ bool SocketPolicyFile::allowsAccessFrom(const URLInfo& url, const URLInfo& to)
  *
  * \return \c true if this policy file is a master policy file, otherwise \c false
  */
-bool SocketPolicyFile::isMaster()
+bool SocketPolicyFile::isMaster() const
 {
 	return url.getPort() == MASTER_PORT;
 }
@@ -1198,11 +1245,11 @@ bool SocketPolicyFile::isIgnoredByMaster()
 		SocketPolicyFile* master = getMasterPolicyFile();
 		getSys()->securityManager->loadSocketPolicyFile(master);
 		//Master policy file found and valid and has a site-control entry
-		if(master->isValid() && master->getSiteControl() != NULL)
+		if(master->isValid())
 		{
-			PolicySiteControl::METAPOLICY permittedPolicies = master->getSiteControl()->getPermittedPolicies();
-			if(permittedPolicies == PolicySiteControl::MASTER_ONLY || 
-			   permittedPolicies == PolicySiteControl::NONE)
+			PolicyFile::METAPOLICY permittedPolicies = master->getMetaPolicy();
+			if(permittedPolicies == PolicyFile::MASTER_ONLY || 
+			   permittedPolicies == PolicyFile::NONE)
 				return true;
 		}
 	}
@@ -1289,23 +1336,31 @@ void SocketPolicyFile::getParserType(CrossDomainPolicy::POLICYFILETYPE& parserTy
  * \param _permittedPolicies The value of the permitted-cross-domain-policies attribute.
  * \see URLPolicyFile::load()
  */
-PolicySiteControl::PolicySiteControl(PolicyFile* _file, const string _permittedPolicies):
-	file(_file)
+PolicySiteControl::PolicySiteControl(const string _permittedPolicies)
 {
 	if(_permittedPolicies == "all")
-		permittedPolicies = ALL;
+		permittedPolicies = PolicyFile::ALL;
 	else if(_permittedPolicies == "by-content-type")
-		permittedPolicies = BY_CONTENT_TYPE;
+		permittedPolicies = PolicyFile::BY_CONTENT_TYPE;
 	else if(_permittedPolicies == "by-ftp-filename")
-		permittedPolicies = BY_FTP_FILENAME;
+		permittedPolicies = PolicyFile::BY_FTP_FILENAME;
 	else if(_permittedPolicies == "master-only")
-		permittedPolicies = MASTER_ONLY;
+		permittedPolicies = PolicyFile::MASTER_ONLY;
 	else if(_permittedPolicies == "none")
-		permittedPolicies = NONE;
-	else if(file->getType() == PolicyFile::SOCKET) //Default for SOCKET
-		permittedPolicies = ALL;
-	else //Default for everything except SOCKET
-		permittedPolicies = MASTER_ONLY;
+		permittedPolicies = PolicyFile::NONE;
+	else
+	{
+		LOG(LOG_ERROR, _("SECURITY: Unknown site-policy value: ") << _permittedPolicies);
+		permittedPolicies = PolicyFile::NONE;
+	}
+}
+
+PolicyFile::METAPOLICY PolicySiteControl::defaultSitePolicy(PolicyFile::TYPE policyFileType)
+{
+	if (policyFileType == PolicyFile::SOCKET)
+		return PolicyFile::ALL;
+	else
+		return PolicyFile::MASTER_ONLY;
 }
 
 /**
@@ -1343,7 +1398,6 @@ PolicyAllowAccessFrom::PolicyAllowAccessFrom(PolicyFile* _file, const string _do
 		size_t dashPos;
 		string startPortStr;
 		string endPortStr;
-		istringstream is;
 		uint16_t startPort;
 		uint16_t endPort;
 		do
