@@ -17,8 +17,4 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#ifdef _MSC_VER
-#pragma pack(pop)
-#endif
-
 #undef PACKED
