@@ -17,7 +17,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#include "config.h"
+#include "parsing/config.h"
 #include "compat.h"
 
 using namespace lightspark;
