@@ -17,7 +17,7 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
-#include "fastpaths.h"
+#include "platforms/fastpaths.h"
 #include <cinttypes>
 
 extern "C"
