@@ -17,7 +17,7 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
-#include "fastpaths.h"
+#include "platforms/fastpaths.h"
 #include <inttypes.h>
 
 void lightspark::fastYUV420ChannelsToYUV0Buffer(uint8_t* y, uint8_t* u, uint8_t* v, uint8_t* out, uint32_t width, uint32_t height)
