@@ -23,7 +23,7 @@
 // Implementation of Netscape entry points (NPN_*)
 //
 #include "compat.h"
-#include "npplat.h"
+#include "plugin/include/npplat.h"
 #include <cstdio>
 #include <cstdlib>
 
