@@ -28,7 +28,11 @@
 #include <llvm/ExecutionEngine/JIT.h>
 #include <llvm/PassManager.h>
 #include <llvm/LLVMContext.h>
-#include <llvm/Target/TargetData.h>
+#ifdef HAVE_DATALAYOUT_H
+#  include <llvm/DataLayout.h>
+#else
+#  include <llvm/Target/TargetData.h>
+#endif
 #ifdef HAVE_SUPPORT_TARGETSELECT_H
 #include <llvm/Support/TargetSelect.h>
 #else
@@ -70,6 +74,7 @@
 #include "scripting/flash/errors/flasherrors.h"
 #include "scripting/flash/text/flashtext.h"
 #include "scripting/flash/text/flashtextengine.h"
+#include "scripting/flash/ui/Keyboard.h"
 #include "scripting/class.h"
 #include "exceptions.h"
 #include "scripting/abc.h"
@@ -266,6 +271,8 @@ void ABCVm::registerClasses()
 	builtin->registerBuiltin("BitmapFilter","flash.filters",Class<BitmapFilter>::getRef());
 	builtin->registerBuiltin("DropShadowFilter","flash.filters",Class<DropShadowFilter>::getRef());
 	builtin->registerBuiltin("GlowFilter","flash.filters",Class<GlowFilter>::getRef());
+	builtin->registerBuiltin("GradientGlowFilter","flash.filters",
+			Class<ASObject>::getStubClass(QName("GradientGlowFilter","flash.filters")));
 	builtin->registerBuiltin("BevelFilter","flash.filters",
 			Class<ASObject>::getStubClass(QName("BevelFilter","flash.filters")));
 	builtin->registerBuiltin("ColorMatrixFilter","flash.filters",
@@ -390,7 +397,9 @@ void ABCVm::registerClasses()
 	builtin->registerBuiltin("StageVideoAvailability","flash.media",Class<StageVideoAvailability>::getRef());
 	builtin->registerBuiltin("VideoStatus","flash.media",Class<VideoStatus>::getRef());
 
-	builtin->registerBuiltin("Keyboard","flash.ui",Class<ASObject>::getStubClass(QName("Keyboard","flash.ui")));
+	builtin->registerBuiltin("Keyboard","flash.ui",Class<Keyboard>::getRef());
+	builtin->registerBuiltin("KeyboardType","flash.ui",Class<KeyboardType>::getRef());
+	builtin->registerBuiltin("KeyLocation","flash.ui",Class<KeyLocation>::getRef());
 	builtin->registerBuiltin("ContextMenu","flash.ui",Class<ASObject>::getStubClass(QName("ContextMenu","flash.ui")));
 	builtin->registerBuiltin("ContextMenuItem","flash.ui",Class<ASObject>::getStubClass(QName("ContextMenuItem","flash.ui")));
 
@@ -1454,7 +1463,11 @@ void ABCVm::Run(ABCVm* th)
 		assert_and_throw(th->ex);
 
 		th->FPM=new llvm::FunctionPassManager(th->module);
+#ifdef HAVE_DATALAYOUT_H
+		th->FPM->add(new llvm::DataLayout(*th->ex->getDataLayout()));
+#else
 		th->FPM->add(new llvm::TargetData(*th->ex->getTargetData()));
+#endif
 #ifdef EXPENSIVE_DEBUG
 		//This is pretty heavy, do not enable in release
 		th->FPM->add(llvm::createVerifierPass());
