@@ -17,12 +17,12 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#ifndef ARGCONV_H_
-#define ARGCONV_H_
+#ifndef SCRIPTING_ARGCONV_H
+#define SCRIPTING_ARGCONV_H 1
 
-#include "toplevel/toplevel.h"
-#include "toplevel/ASString.h"
-#include "toplevel/Boolean.h"
+#include "scripting/toplevel/toplevel.h"
+#include "scripting/toplevel/ASString.h"
+#include "scripting/toplevel/Boolean.h"
 
 /* Usage of ARG_UNPACK:
  * You have to use it within a ASFUNCTIONBODY() { }, because it uses the implicit arguments 'args' and 'argslen'.
@@ -253,4 +253,4 @@ public:
 };
 
 }
-#endif /* ARGCONV_H_ */
+#endif /* SCRIPTING_ARGCONV_H */
