@@ -44,7 +44,11 @@ friend class Transform;
 friend class ParseThread;
 friend std::ostream& operator<<(std::ostream& s, const DisplayObject& r);
 public:
-	enum HIT_TYPE { GENERIC_HIT, DOUBLE_CLICK };
+	enum HIT_TYPE { GENERIC_HIT, // point is over the object
+                        GENERIC_HIT_INVISIBLE, // ...even if the object is invisible
+			MOUSE_CLICK, // point over the object and mouseEnabled
+			DOUBLE_CLICK // point over the object and doubleClickEnabled
+		      };
 private:
 	ASPROPERTY_GETTER_SETTER(_NR<AccessibilityProperties>,accessibilityProperties);
 	static ATOMIC_INT32(instanceCount);
@@ -102,11 +106,12 @@ protected:
 	{
 		throw RunTimeException("DisplayObject::boundsRect: Derived class must implement this!");
 	}
+	bool boundsRectGlobal(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const;
 	virtual void renderImpl(RenderContext& ctxt) const
 	{
 		throw RunTimeException("DisplayObject::renderImpl: Derived class must implement this!");
 	}
-	virtual _NR<InteractiveObject> hitTestImpl(_NR<InteractiveObject> last, number_t x, number_t y, HIT_TYPE type)
+	virtual _NR<DisplayObject> hitTestImpl(_NR<DisplayObject> last, number_t x, number_t y, HIT_TYPE type)
 	{
 		throw RunTimeException("DisplayObject::hitTestImpl: Derived class must implement this!");
 	}
@@ -151,7 +156,7 @@ public:
 	}
 	void Render(RenderContext& ctxt);
 	bool getBounds(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax, const MATRIX& m) const;
-	_NR<InteractiveObject> hitTest(_NR<InteractiveObject> last, number_t x, number_t y, HIT_TYPE type);
+	_NR<DisplayObject> hitTest(_NR<DisplayObject> last, number_t x, number_t y, HIT_TYPE type);
 	virtual void setOnStage(bool staged);
 	bool isOnStage() const { return onStage; }
 	bool isMask() const { return !maskOf.isNull(); }
@@ -206,6 +211,8 @@ public:
 	ASFUNCTION(_getTransform);
 	ASFUNCTION(localToGlobal);
 	ASFUNCTION(globalToLocal);
+	ASFUNCTION(hitTestObject);
+	ASFUNCTION(hitTestPoint);
 };
 };
 #endif /* SCRIPTING_FLASH_DISPLAY_DISPLAYOBJECT_H */
