@@ -218,18 +218,20 @@ ASFUNCTIONBODY(FocusEvent,_constructor)
 	return NULL;
 }
 
-MouseEvent::MouseEvent(Class_base* c):Event(c, "mouseEvent"), localX(0), localY(0), stageX(0), stageY(0), relatedObject(NullRef)
+MouseEvent::MouseEvent(Class_base* c)
+ : Event(c, "mouseEvent"), modifiers(0), delta(1), localX(0), localY(0), stageX(0), stageY(0), relatedObject(NullRef)
 {
 }
 
-MouseEvent::MouseEvent(Class_base* c, const tiny_string& t, number_t lx, number_t ly, bool b, _NR<InteractiveObject> relObj):Event(c,t,b),
-	 localX(lx), localY(ly), stageX(0), stageY(0), relatedObject(relObj)
+MouseEvent::MouseEvent(Class_base* c, const tiny_string& t, number_t lx, number_t ly,
+		       bool b, unsigned int buttonState, _NR<InteractiveObject> relObj, int32_t _delta)
+  : Event(c,t,b), modifiers(buttonState), delta(_delta), localX(lx), localY(ly), stageX(0), stageY(0), relatedObject(relObj)
 {
 }
 
 Event* MouseEvent::cloneImpl() const
 {
-	return Class<MouseEvent>::getInstanceS(type,localX,localY,bubbles,relatedObject);
+	return Class<MouseEvent>::getInstanceS(type,localX,localY,bubbles,modifiers,relatedObject,delta);
 }
 
 ProgressEvent::ProgressEvent(Class_base* c):Event(c, "progress",false),bytesLoaded(0),bytesTotal(0)
@@ -301,12 +303,16 @@ void MouseEvent::sinit(Class_base* c)
 	c->setVariableByQName("ROLL_OVER","",Class<ASString>::getInstanceS("rollOver"),DECLARED_TRAIT);
 	c->setVariableByQName("ROLL_OUT","",Class<ASString>::getInstanceS("rollOut"),DECLARED_TRAIT);
 
-
 	REGISTER_GETTER(c,relatedObject);
 	REGISTER_GETTER(c,stageX);
 	REGISTER_GETTER(c,stageY);
 	REGISTER_GETTER_SETTER(c,localX);
 	REGISTER_GETTER_SETTER(c,localY);
+	REGISTER_GETTER_SETTER(c,altKey);
+	REGISTER_GETTER_SETTER(c,buttonDown);
+	REGISTER_GETTER_SETTER(c,ctrlKey);
+	REGISTER_GETTER_SETTER(c,shiftKey);
+	REGISTER_GETTER_SETTER(c,delta);
 }
 
 ASFUNCTIONBODY(MouseEvent,_constructor)
@@ -320,6 +326,26 @@ ASFUNCTIONBODY(MouseEvent,_constructor)
 		th->localY=args[4]->toNumber();
 	if(argslen>=6)
 		th->relatedObject=ArgumentConversion< _NR<InteractiveObject> >::toConcrete(args[5]);
+	if(argslen>=7)
+		if (ArgumentConversion<bool>::toConcrete(args[6]))
+			th->modifiers |= GDK_CONTROL_MASK;
+	if(argslen>=8)
+		if (ArgumentConversion<bool>::toConcrete(args[7]))
+			th->modifiers |= GDK_MOD1_MASK;
+	if(argslen>=9)
+		if (ArgumentConversion<bool>::toConcrete(args[8]))
+			th->modifiers |= GDK_SHIFT_MASK;
+	if(argslen>=10)
+		if (ArgumentConversion<bool>::toConcrete(args[9]))
+			th->modifiers |= GDK_BUTTON1_MASK;
+	if(argslen>=11)
+		th->delta=args[10]->toInt();
+	// TODO: args[11] = command
+	if(argslen>=13)
+		if (ArgumentConversion<bool>::toConcrete(args[12]))
+			th->modifiers |= GDK_CONTROL_MASK;
+	// TODO: args[13] = clickCount
+
 	return NULL;
 }
 
@@ -328,6 +354,7 @@ ASFUNCTIONBODY_GETTER(MouseEvent,localX);
 ASFUNCTIONBODY_GETTER(MouseEvent,localY);
 ASFUNCTIONBODY_GETTER(MouseEvent,stageX);
 ASFUNCTIONBODY_GETTER(MouseEvent,stageY);
+ASFUNCTIONBODY_GETTER_SETTER(MouseEvent,delta);
 
 ASFUNCTIONBODY(MouseEvent,_setter_localX)
 {
@@ -363,6 +390,71 @@ ASFUNCTIONBODY(MouseEvent,_setter_localY)
 	return NULL; 
 }
 
+ASFUNCTIONBODY(MouseEvent,_getter_buttonDown)
+{
+	MouseEvent* th=static_cast<MouseEvent*>(obj);
+	return abstract_b(th->modifiers & GDK_BUTTON1_MASK);
+}
+
+ASFUNCTIONBODY(MouseEvent,_setter_buttonDown)
+{
+	MouseEvent* th=static_cast<MouseEvent*>(obj);
+	th->modifiers |= GDK_BUTTON1_MASK;
+	return NULL;
+}
+
+ASFUNCTIONBODY(MouseEvent,_getter_altKey)
+{
+	MouseEvent* th=static_cast<MouseEvent*>(obj);
+	return abstract_b(th->modifiers & GDK_MOD1_MASK);
+}
+
+ASFUNCTIONBODY(MouseEvent,_setter_altKey)
+{
+	MouseEvent* th=static_cast<MouseEvent*>(obj);
+	th->modifiers |= GDK_MOD1_MASK;
+	return NULL;
+}
+
+ASFUNCTIONBODY(MouseEvent,_getter_controlKey)
+{
+	MouseEvent* th=static_cast<MouseEvent*>(obj);
+	return abstract_b(th->modifiers & GDK_CONTROL_MASK);
+}
+
+ASFUNCTIONBODY(MouseEvent,_setter_controlKey)
+{
+	MouseEvent* th=static_cast<MouseEvent*>(obj);
+	th->modifiers |= GDK_CONTROL_MASK;
+	return NULL;
+}
+
+ASFUNCTIONBODY(MouseEvent,_getter_ctrlKey)
+{
+	MouseEvent* th=static_cast<MouseEvent*>(obj);
+	return abstract_b(th->modifiers & GDK_CONTROL_MASK);
+}
+
+ASFUNCTIONBODY(MouseEvent,_setter_ctrlKey)
+{
+	MouseEvent* th=static_cast<MouseEvent*>(obj);
+	th->modifiers |= GDK_CONTROL_MASK;
+	return NULL;
+}
+
+ASFUNCTIONBODY(MouseEvent,_getter_shiftKey)
+{
+	MouseEvent* th=static_cast<MouseEvent*>(obj);
+	return abstract_b(th->modifiers & GDK_SHIFT_MASK);
+}
+
+ASFUNCTIONBODY(MouseEvent,_setter_shiftKey)
+{
+	MouseEvent* th=static_cast<MouseEvent*>(obj);
+	th->modifiers |= GDK_SHIFT_MASK;
+	return NULL;
+}
+
 void MouseEvent::buildTraits(ASObject* o)
 {
 	//TODO: really handle local[XY]
@@ -662,7 +754,6 @@ void NetStatusEvent::sinit(Class_base* c)
 
 ASFUNCTIONBODY(NetStatusEvent,_constructor)
 {
-	NetStatusEvent* th=Class<NetStatusEvent>::cast(obj);
 	//Also call the base class constructor, using only the first arguments
 	uint32_t baseClassArgs=imin(argslen,3);
 	Event::_constructor(obj,args,baseClassArgs);
@@ -728,7 +819,8 @@ ASFUNCTIONBODY(FullScreenEvent,_constructor)
 	return NULL;
 }
 
-KeyboardEvent::KeyboardEvent(Class_base* c):Event(c, "keyboardEvent")
+KeyboardEvent::KeyboardEvent(Class_base* c, tiny_string _type, uint32_t _charcode, uint32_t _keycode, unsigned _modifiers)
+  : Event(c, _type), modifiers(_modifiers), charCode(_charcode), keyCode(_keycode), keyLocation(0)
 {
 }
 
@@ -737,17 +829,136 @@ void KeyboardEvent::sinit(Class_base* c)
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
 	c->setSuper(Class<Event>::getRef());
 
+	REGISTER_GETTER_SETTER(c, altKey);
+	REGISTER_GETTER_SETTER(c, charCode);
+	REGISTER_GETTER_SETTER(c, commandKey);
+	REGISTER_GETTER_SETTER(c, controlKey);
+	REGISTER_GETTER_SETTER(c, ctrlKey);
+	REGISTER_GETTER_SETTER(c, keyCode);
+	REGISTER_GETTER_SETTER(c, keyLocation);
+	REGISTER_GETTER_SETTER(c, shiftKey);
 	c->setVariableByQName("KEY_DOWN","",Class<ASString>::getInstanceS("keyDown"),DECLARED_TRAIT);
 	c->setVariableByQName("KEY_UP","",Class<ASString>::getInstanceS("keyUp"),DECLARED_TRAIT);
 }
 
 ASFUNCTIONBODY(KeyboardEvent,_constructor)
 {
+	KeyboardEvent* th=static_cast<KeyboardEvent*>(obj);
+
 	uint32_t baseClassArgs=imin(argslen,3);
 	Event::_constructor(obj,args,baseClassArgs);
+
+	if(argslen > 3) {
+		th->charCode = args[3]->toUInt();
+	}
+	if(argslen > 4) {
+		th->keyCode = args[4]->toUInt();
+	}
+	if(argslen > 5) {
+		th->keyLocation = args[5]->toUInt();
+	}
+	if(argslen > 6) {
+		if (ArgumentConversion<bool>::toConcrete(args[6]))
+			th->modifiers |= GDK_CONTROL_MASK;
+	}
+	if(argslen > 7) {
+		if (ArgumentConversion<bool>::toConcrete(args[7]))
+			th->modifiers |= GDK_MOD1_MASK;
+	}
+	if(argslen > 8) {
+		if (ArgumentConversion<bool>::toConcrete(args[8]))
+			th->modifiers |= GDK_SHIFT_MASK;
+	}
+	if(argslen > 9) {
+		if (ArgumentConversion<bool>::toConcrete(args[9]))
+			th->modifiers |= GDK_CONTROL_MASK;
+	}
+	// args[10] (commandKeyValue) is only supported on Max OSX
+
 	return NULL;
 }
 
+ASFUNCTIONBODY_GETTER_SETTER(KeyboardEvent, charCode);
+ASFUNCTIONBODY_GETTER_SETTER(KeyboardEvent, keyCode);
+ASFUNCTIONBODY_GETTER_SETTER(KeyboardEvent, keyLocation);
+
+ASFUNCTIONBODY(KeyboardEvent, _getter_altKey)
+{
+	KeyboardEvent* th=static_cast<KeyboardEvent*>(obj);
+	return abstract_b(th->modifiers & GDK_MOD1_MASK);
+}
+
+ASFUNCTIONBODY(KeyboardEvent, _setter_altKey)
+{
+	KeyboardEvent* th=static_cast<KeyboardEvent*>(obj);
+	th->modifiers |= GDK_MOD1_MASK;
+	return NULL;
+}
+
+ASFUNCTIONBODY(KeyboardEvent, _getter_commandKey)
+{
+	// Supported only on OSX
+	return abstract_b(false);
+}
+
+ASFUNCTIONBODY(KeyboardEvent, _setter_commandKey)
+{
+	// Supported only on OSX
+	return NULL;
+}
+
+ASFUNCTIONBODY(KeyboardEvent, _getter_controlKey)
+{
+	KeyboardEvent* th=static_cast<KeyboardEvent*>(obj);
+	return abstract_b(th->modifiers & GDK_CONTROL_MASK);
+}
+
+ASFUNCTIONBODY(KeyboardEvent, _setter_controlKey)
+{
+	KeyboardEvent* th=static_cast<KeyboardEvent*>(obj);
+	th->modifiers |= GDK_CONTROL_MASK;
+	return NULL;
+}
+
+ASFUNCTIONBODY(KeyboardEvent, _getter_ctrlKey)
+{
+	KeyboardEvent* th=static_cast<KeyboardEvent*>(obj);
+	return abstract_b(th->modifiers & GDK_CONTROL_MASK);
+}
+
+ASFUNCTIONBODY(KeyboardEvent, _setter_ctrlKey)
+{
+	KeyboardEvent* th=static_cast<KeyboardEvent*>(obj);
+	th->modifiers |= GDK_CONTROL_MASK;
+	return NULL;
+}
+
+ASFUNCTIONBODY(KeyboardEvent, _getter_shiftKey)
+{
+	KeyboardEvent* th=static_cast<KeyboardEvent*>(obj);
+	return abstract_b(th->modifiers & GDK_SHIFT_MASK);
+}
+
+ASFUNCTIONBODY(KeyboardEvent, _setter_shiftKey)
+{
+	KeyboardEvent* th=static_cast<KeyboardEvent*>(obj);
+	th->modifiers |= GDK_SHIFT_MASK;
+	return NULL;
+}
+
+Event* KeyboardEvent::cloneImpl() const
+{
+	KeyboardEvent *cloned = Class<KeyboardEvent>::getInstanceS();
+	cloned->type = type;
+	cloned->bubbles = bubbles;
+	cloned->cancelable = cancelable;
+	cloned->modifiers = modifiers;
+	cloned->charCode = charCode;
+	cloned->keyCode = keyCode;
+	cloned->keyLocation = keyLocation;
+	return cloned;
+}
+
 TextEvent::TextEvent(Class_base* c,const tiny_string& t):Event(c,t)
 {
 }
