@@ -43,12 +43,16 @@ void XMLNode::sinit(Class_base* c)
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
 	c->setSuper(Class<ASObject>::getRef());
 	c->setDeclaredMethodByQName("toString","",Class<IFunction>::getFunction(_toString),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("firstChild","",Class<IFunction>::getFunction(XMLNode::firstChild),GETTER_METHOD,true);
-	c->setDeclaredMethodByQName("childNodes","",Class<IFunction>::getFunction(XMLNode::childNodes),GETTER_METHOD,true);
 	c->setDeclaredMethodByQName("attributes","",Class<IFunction>::getFunction(attributes),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("childNodes","",Class<IFunction>::getFunction(XMLNode::childNodes),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("firstChild","",Class<IFunction>::getFunction(XMLNode::firstChild),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("lastChild","",Class<IFunction>::getFunction(lastChild),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("nextSibling","",Class<IFunction>::getFunction(nextSibling),GETTER_METHOD,true);
 	c->setDeclaredMethodByQName("nodeType","",Class<IFunction>::getFunction(_getNodeType),GETTER_METHOD,true);
 	c->setDeclaredMethodByQName("nodeName","",Class<IFunction>::getFunction(_getNodeName),GETTER_METHOD,true);
 	c->setDeclaredMethodByQName("nodeValue","",Class<IFunction>::getFunction(_getNodeValue),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("parentNode","",Class<IFunction>::getFunction(parentNode),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("previousSibling","",Class<IFunction>::getFunction(previousSibling),GETTER_METHOD,true);
 }
 
 void XMLNode::buildTraits(ASObject* o)
@@ -77,9 +81,9 @@ ASFUNCTIONBODY(XMLNode,firstChild)
 {
 	XMLNode* th=Class<XMLNode>::cast(obj);
 	assert_and_throw(argslen==0);
-	if(th->node==NULL) //We assume NULL node is like empty node
+	//We assume NULL node is like empty node
+	if(th->node==NULL || th->node->cobj()->type==XML_TEXT_NODE)
 		return getSys()->getNullRef();
-	assert_and_throw(th->node->cobj()->type!=XML_TEXT_NODE);
 	const xmlpp::Node::NodeList& children=th->node->get_children();
 	if(children.empty())
 		return getSys()->getNullRef();
@@ -88,6 +92,21 @@ ASFUNCTIONBODY(XMLNode,firstChild)
 	return Class<XMLNode>::getInstanceS(th->root,newNode);
 }
 
+ASFUNCTIONBODY(XMLNode,lastChild)
+{
+	XMLNode* th=Class<XMLNode>::cast(obj);
+	assert_and_throw(argslen==0);
+	//We assume NULL node is like empty node
+	if(th->node==NULL || th->node->cobj()->type==XML_TEXT_NODE)
+		return getSys()->getNullRef();
+	const xmlpp::Node::NodeList& children=th->node->get_children();
+	if(children.empty())
+		return getSys()->getNullRef();
+	xmlpp::Node* newNode=children.back();
+	assert_and_throw(!th->root.isNull());
+	return Class<XMLNode>::getInstanceS(th->root,newNode);
+}
+
 ASFUNCTIONBODY(XMLNode,childNodes)
 {
 	XMLNode* th=Class<XMLNode>::cast(obj);
@@ -133,6 +152,54 @@ ASFUNCTIONBODY(XMLNode,attributes)
 	return ret;
 }
 
+xmlpp::Node *XMLNode::getParentNode()
+{
+	if (!node)
+		return NULL;
+
+	xmlpp::Node *parent = node->get_parent();
+	if (parent)
+		return parent;
+	else
+		return NULL;
+}
+
+ASFUNCTIONBODY(XMLNode,parentNode)
+{
+	XMLNode* th=Class<XMLNode>::cast(obj);
+	xmlpp::Node *parent = th->getParentNode();
+	if (parent)
+		return Class<XMLNode>::getInstanceS(th->root, parent);
+	else
+		return getSys()->getNullRef();
+}
+
+ASFUNCTIONBODY(XMLNode,nextSibling)
+{
+	XMLNode* th=Class<XMLNode>::cast(obj);
+	if(th->node==NULL)
+		return getSys()->getNullRef();
+
+	xmlpp::Node *sibling = th->node->get_next_sibling();
+	if (sibling)
+		return Class<XMLNode>::getInstanceS(th->root, sibling);
+	else
+		return getSys()->getNullRef();
+}
+
+ASFUNCTIONBODY(XMLNode,previousSibling)
+{
+	XMLNode* th=Class<XMLNode>::cast(obj);
+	if(th->node==NULL)
+		return getSys()->getNullRef();
+
+	xmlpp::Node *sibling = th->node->get_previous_sibling();
+	if (sibling)
+		return Class<XMLNode>::getInstanceS(th->root, sibling);
+	else
+		return getSys()->getNullRef();
+}
+
 ASFUNCTIONBODY(XMLNode,_getNodeType)
 {
 	XMLNode* th=Class<XMLNode>::cast(obj);
@@ -185,7 +252,7 @@ tiny_string XMLNode::toString_priv(xmlpp::Node *outputNode)
 }
 
 XMLDocument::XMLDocument(Class_base* c, tiny_string s)
- : XMLNode(c),rootNode(NULL)
+  : XMLNode(c),rootNode(NULL),ignoreWhite(false)
 {
 	if(!s.empty())
 	{
@@ -200,8 +267,11 @@ void XMLDocument::sinit(Class_base* c)
 	c->setDeclaredMethodByQName("parseXML","",Class<IFunction>::getFunction(parseXML),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("toString","",Class<IFunction>::getFunction(_toString),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("firstChild","",Class<IFunction>::getFunction(XMLDocument::firstChild),GETTER_METHOD,true);
+	REGISTER_GETTER_SETTER(c, ignoreWhite);
 }
 
+ASFUNCTIONBODY_GETTER_SETTER(XMLDocument, ignoreWhite);
+
 void XMLDocument::buildTraits(ASObject* o)
 {
 }
@@ -228,7 +298,7 @@ void XMLDocument::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& str
 
 void XMLDocument::parseXMLImpl(const string& str)
 {
-	rootNode=buildFromString(str);
+	rootNode=buildFromString(str, ignoreWhite);
 }
 
 ASFUNCTIONBODY(XMLDocument,_toString)
