@@ -17,8 +17,8 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#ifndef TOPLEVEL_ARRAY_H
-#define TOPLEVEL_ARRAY_H
+#ifndef SCRIPTING_TOPLEVEL_ARRAY_H
+#define SCRIPTING_TOPLEVEL_ARRAY_H 1
 
 #include "asobject.h"
 
@@ -29,22 +29,31 @@ enum DATA_TYPE {DATA_OBJECT=0,DATA_INT};
 
 struct data_slot
 {
-	DATA_TYPE type;
 	union
 	{
 		ASObject* data;
 		int32_t data_i;
 	};
-	explicit data_slot(ASObject* o,DATA_TYPE t=DATA_OBJECT):type(t),data(o){}
-	data_slot():type(DATA_OBJECT),data(NULL){}
-	explicit data_slot(int32_t i):type(DATA_INT),data_i(i){}
+	DATA_TYPE type;
+	explicit data_slot(ASObject* o):data(o),type(DATA_OBJECT){}
+	data_slot():data(NULL),type(DATA_OBJECT){}
+	explicit data_slot(int32_t i):data_i(i),type(DATA_INT){}
 };
+struct sorton_field
+{
+	bool isNumeric;
+	bool isCaseInsensitive;
+	bool isDescending;
+	multiname fieldname;
+	sorton_field(const multiname& sortfieldname):isNumeric(false),isCaseInsensitive(false),isDescending(false),fieldname(sortfieldname){}
+};
+
 
 class Array: public ASObject
 {
 friend class ABCVm;
 protected:
-	uint64_t currentsize;
+	uint32_t currentsize;
 	typedef std::map<uint32_t,data_slot,std::less<uint32_t>,
 		reporter_allocator<std::pair<const uint32_t, data_slot>>> arrayType;
 	arrayType data;
@@ -70,6 +79,14 @@ private:
 		sortComparatorWrapper(IFunction* c):comparator(c){}
 		bool operator()(const data_slot& d1, const data_slot& d2);
 	};
+	class sortOnComparator
+	{
+	private:
+		std::vector<sorton_field> fields;
+	public:
+		sortOnComparator(const std::vector<sorton_field>& sf):fields(sf){}
+		bool operator()(const data_slot& d1, const data_slot& d2);
+	};
 	tiny_string toString_priv() const;
 	int capIndex(int i) const;
 public:
@@ -128,7 +145,7 @@ public:
 	void push(_R<ASObject> o)
 	{
 		o->incRef();
-		data[currentsize] = data_slot(o.getPtr(),DATA_OBJECT);
+		data[currentsize] = data_slot(o.getPtr());
 		currentsize++;
 	}
 	void resize(uint64_t n);
@@ -137,7 +154,7 @@ public:
 	void setVariableByMultiname(const multiname& name, ASObject* o, CONST_ALLOWED_FLAG allowConst);
 	bool deleteVariableByMultiname(const multiname& name);
 	void setVariableByMultiname_i(const multiname& name, int32_t value);
-	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic);
+	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic, bool considerPrototype);
 	tiny_string toString();
 	uint32_t nextNameIndex(uint32_t cur_index);
 	_R<ASObject> nextName(uint32_t index);
@@ -150,4 +167,4 @@ public:
 
 
 }
-#endif
+#endif /* SCRIPTING_TOPLEVEL_ARRAY_H */
