@@ -17,23 +17,21 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#include "Boolean.h"
-#include "toplevel.h"
-#include "class.h"
-#include "argconv.h"
+#include "scripting/toplevel/Boolean.h"
+#include "scripting/toplevel/toplevel.h"
+#include "scripting/class.h"
+#include "scripting/argconv.h"
 #include "parsing/amf3_generator.h"
 
 using namespace lightspark;
 using namespace std;
 
-SET_NAMESPACE("");
-REGISTER_CLASS_NAME(Boolean);
-
 Boolean* lightspark::abstract_b(bool v)
 {
-	Boolean* b = Class<Boolean>::getInstanceS();
-	b->val = v;
-	return b;
+	if(v==true)
+		return getSys()->getTrueRef();
+	else
+		return getSys()->getFalseRef();
 }
 
 /* implements ecma3's ToBoolean() operation, see section 9.2, but returns the value instead of an Boolean object */
@@ -81,15 +79,18 @@ void Boolean::sinit(Class_base* c)
 ASFUNCTIONBODY(Boolean,_constructor)
 {
 	Boolean* th=static_cast<Boolean*>(obj);
-	_NR<ASObject> o;
-	ARG_UNPACK (o,_MNR(getSys()->getUndefinedRef()));
-	th->val=Boolean_concrete(o.getPtr());
+	if(argslen==0)
+	{
+		//No need to handle default argument. The object is initialized to false anyway
+		return NULL;
+	}
+	th->val=Boolean_concrete(args[0]);
 	return NULL;
 }
 
 ASFUNCTIONBODY(Boolean,_toString)
 {
-	if(Class<Boolean>::getClass()->prototype == obj) //See ECMA 15.6.4
+	if(Class<Boolean>::getClass()->prototype->getObj() == obj) //See ECMA 15.6.4
 		return Class<ASString>::getInstanceS("false");
 
 	if(!obj->is<Boolean>())
@@ -101,7 +102,7 @@ ASFUNCTIONBODY(Boolean,_toString)
 
 ASFUNCTIONBODY(Boolean,_valueOf)
 {
-	if(Class<Boolean>::getClass()->prototype == obj)
+	if(Class<Boolean>::getClass()->prototype->getObj() == obj)
 		return abstract_b(false);
 
 	if(!obj->is<Boolean>())
