@@ -17,8 +17,8 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#ifndef ERROR_H_
-#define ERROR_H_
+#ifndef SCRIPTING_TOPLEVEL_ERROR_H
+#define SCRIPTING_TOPLEVEL_ERROR_H 1
 
 #include "asobject.h"
 
@@ -27,13 +27,13 @@ namespace lightspark
 
 class ASError: public ASObject
 {
+private:
+	ASPROPERTY_GETTER(int32_t, errorID);
+	ASPROPERTY_GETTER_SETTER(tiny_string, name);
 protected:
 	ASPROPERTY_GETTER_SETTER(tiny_string, message);
 	void setErrorID(int32_t id) { errorID=id; }
 	static void errorGenerator(ASError *obj, ASObject* const* args, const unsigned int argslen);
-private:
-	ASPROPERTY_GETTER(int32_t, errorID);
-	ASPROPERTY_GETTER_SETTER(tiny_string, name);
 public:
 	ASError(Class_base* c, const tiny_string& error_message = "", int id = 0, const tiny_string& error_name="Error");
 	ASFUNCTION(_constructor);
@@ -158,4 +158,4 @@ public:
 
 }
 
-#endif /* ERROR_H_ */
+#endif /* SCRIPTING_TOPLEVEL_ERROR_H */
