@@ -233,3 +233,14 @@ bool Integer::fromStringFlashCompatible(const char* cur, int64_t& ret, int radix
 	ret*=multiplier;
 	return true;
 }
+
+int32_t Integer::stringToASInteger(const char* cur, int radix)
+{
+	int64_t value;
+	bool valid=Integer::fromStringFlashCompatible(cur, value, 0);
+
+	if(valid==false || value<INT32_MIN || value>INT32_MAX)
+		return 0;
+	else
+		return static_cast<int32_t>(value);
+}
