@@ -19,6 +19,7 @@
 
 #include <string>
 #include <algorithm>
+#include "backends/security.h"
 #include "scripting/abc.h"
 #include "scripting/flash/events/flashevents.h"
 #include "scripting/flash/utils/flashutils.h"
@@ -31,9 +32,9 @@
 #include "backends/config.h"
 #include "backends/pluginmanager.h"
 #include "backends/rendering.h"
-#include "backends/security.h"
 #include "backends/image.h"
 #include "backends/extscriptobject.h"
+#include "backends/input.h"
 #include "memory_support.h"
 
 #ifdef ENABLE_CURL
@@ -1827,6 +1828,95 @@ void SystemState::getUniqueNamespaceId(const nsNameAndKindImpl& s, uint32_t hint
 	baseId=(it->first.baseId==0xffffffff)?nsId:it->first.baseId;
 }
 
+void SystemState::stageCoordinateMapping(uint32_t windowWidth, uint32_t windowHeight,
+					 int& offsetX, int& offsetY,
+					 float& scaleX, float& scaleY)
+{
+	//Get the size of the content
+	RECT r=mainClip->getFrameSize();
+	r.Xmax/=20;
+	r.Ymax/=20;
+	//Now compute the scalings and offsets
+	switch(scaleMode)
+	{
+		case SystemState::SHOW_ALL:
+			//Compute both scaling
+			scaleX=windowWidth;
+			scaleX/=r.Xmax;
+			scaleY=windowHeight;
+			scaleY/=r.Ymax;
+			//Enlarge with no distortion
+			if(scaleX<scaleY)
+			{
+				//Uniform scaling for Y
+				scaleY=scaleX;
+				//Apply the offset
+				offsetY=(windowHeight-r.Ymax*scaleY)/2;
+				offsetX=0;
+			}
+			else
+			{
+				//Uniform scaling for X
+				scaleX=scaleY;
+				//Apply the offset
+				offsetX=(windowWidth-r.Xmax*scaleX)/2;
+				offsetY=0;
+			}
+			break;
+		case SystemState::NO_BORDER:
+			//Compute both scaling
+			scaleX=windowWidth;
+			scaleX/=r.Xmax;
+			scaleY=windowHeight;
+			scaleY/=r.Ymax;
+			//Enlarge with no distortion
+			if(scaleX>scaleY)
+			{
+				//Uniform scaling for Y
+				scaleY=scaleX;
+				//Apply the offset
+				offsetY=(windowHeight-r.Ymax*scaleY)/2;
+				offsetX=0;
+			}
+			else
+			{
+				//Uniform scaling for X
+				scaleX=scaleY;
+				//Apply the offset
+				offsetX=(windowWidth-r.Xmax*scaleX)/2;
+				offsetY=0;
+			}
+			break;
+		case SystemState::EXACT_FIT:
+			//Compute both scaling
+			scaleX=windowWidth;
+			scaleX/=r.Xmax;
+			scaleY=windowHeight;
+			scaleY/=r.Ymax;
+			offsetX=0;
+			offsetY=0;
+			break;
+		case SystemState::NO_SCALE:
+			scaleX=1;
+			scaleY=1;
+			offsetX=0;
+			offsetY=0;
+			break;
+	}
+}
+
+void SystemState::windowToStageCoordinates(int windowX, int windowY, int& stageX, int& stageY)
+{
+	int offsetX;
+	int offsetY;
+	float scaleX;
+	float scaleY;
+	stageCoordinateMapping(renderThread->windowWidth, renderThread->windowHeight,
+			       offsetX, offsetY, scaleX, scaleY);
+	stageX = (windowX-offsetX)/scaleX;
+	stageY = (windowY-offsetY)/scaleY;
+}
+
 /* This is run in vm's thread context */
 void RootMovieClip::initFrame()
 {
