@@ -805,7 +805,7 @@ public:
 	}
 	number_t getRotation() const
 	{
-		return atan(xx/yx)*180/M_PI;
+		return atan(yx/xx)*180/M_PI;
 	}
 	/*
 	 * Implement flash style premultiply matrix operators
@@ -1100,6 +1100,16 @@ private:
 	SB GreenAddTerm;
 	SB BlueAddTerm;
 	SB AlphaAddTerm;
+public:
+	void getParameters(number_t& redMultiplier, 
+			   number_t& greenMultiplier, 
+			   number_t& blueMultiplier,
+			   number_t& alphaMultiplier,
+			   number_t& redOffset,
+			   number_t& greenOffset,
+			   number_t& blueOffset,
+			   number_t& alphaOffset) const;
+	float transformedAlpha(float alpha) const;
 };
 
 class CXFORM
